#!/usr/bin/perl



use strict;
#use KCatch;

use Digest::SHA1;
use Time::HiRes;

require 'data.pl';
require 'string.pl';
require 'configuration.pl';



our %argument = &string::splitQueryString( $ENV{'QUERY_STRING'} );
our $id = Digest::SHA1::sha1_base64( $ENV{'HTTP_USER_AGENT'} . $ENV{'REMOTE_HOST'} . $argument{'id'} );
$ENV{'TZ'} = $main::configuration{'timeZone'};
chdir( $main::configuration{'dataDirectory'} );



# 入室
if ( $argument{'mode'} eq 'login' )
{
	my $data;
	my $now = Time::HiRes::time();

	# データ読み込み
	{
		my $count = 0;
		while ( !( $data = new Data( $main::configuration{'dataFile'} ) ) )
		{
			Time::HiRes::sleep( $main::configuration{'lockInterval'} );
			if ( ++$count > $main::configuration{'retry'} )
			{
				print( "Content-type: text/plain; charset=UTF-8\n\n入室処理失敗 - ロックができませんでした．" );
				exit;
			}
		}
	}

	# ユーザーID確認
	if ( !exists( $data->{'users'}->{$id} ) )
	{
		print( "Content-type: text/plain; charset=UTF-8\n\n入室処理失敗 - ユーザーIDが不整合です．" );
		exit;
	}

	# ユーザー名確認
	if ( !$argument{'name'} ) 
	{
		print( "Content-type: text/plain; charset=UTF-8\n\n入室処理失敗 - ユーザー名が指定されていません．" );
		exit;
	}

	# ◆切断済みは入室できない
	if ( $data->{'users'}->{$id}->{'name'} eq '◆切断済み' )
	{
		print( "Content-type: text/plain; charset=UTF-8\n\n入室処理失敗 - 切断済みです．ブラウザでリロードしてください．" );
		exit;
	}

	# ◆閲覧者でなければ入室できない
	if ( $data->{'users'}->{$id}->{'name'} ne '◆閲覧者' )
	{
		print( "Content-type: text/plain; charset=UTF-8\n\n入室処理失敗 - 既に入室されています．" );
		exit;
	}

	# ユーザー名変換（トリップ処理など）
	$argument{'name'} = &string::trip( $argument{'name'} );

	# ユーザー名重複確認
	foreach ( keys( %{ $data->{'users'} } ) )
	{
		if ( $argument{'name'} eq $data->{'users'}->{$_}->{'name'} )
		{
			print( "Content-type: text/plain; charset=UTF-8\n\n入室処理失敗 - このユーザー名は既に使われています．" );
			exit;
		}
	}

	# ユーザー名書き換え
	$data->{'users'}->{$id}->{'name'} = $argument{'name'};

	# 入室メッセージ書き込み
	{
		my $newMessage = {
			'speaker' => "<span title=\"UA: $ENV{'HTTP_USER_AGENT'} , Host: $ENV{'REMOTE_HOST'}\">$data->{'users'}->{$id}->{'name'}</span>" ,
			'statement' => '<span class="system_message">入室</span>' ,
			'postedAt' => &string::yymmddhhMMss()
		};
		push( @{ $data->{'conversation'} } , $newMessage );
	}

	# 更新時刻書き換え
	$data->{'lastUpdate'} = $now;

	print( "Content-type: text/plain; charset=UTF-8\n\n入室処理完了" );
	exit;
}



# 退室
if ( $argument{'mode'} eq 'logout' )
{
	my $data;
	my $now = Time::HiRes::time();

	# データ読み込み
	{
		my $count = 0;
		while ( !( $data = new Data( $main::configuration{'dataFile'} ) ) )
		{
			Time::HiRes::sleep( $main::configuration{'lockInterval'} );
			if ( ++$count > $main::configuration{'retry'} )
			{
				print( "Content-type: text/plain; charset=UTF-8\n\n退室処理失敗 - ロックができませんでした．" );
				exit;
			}
		}
	}

	# ユーザーID確認
	if ( !exists( $data->{'users'}->{$id} ) )
	{
		print( "Content-type: text/plain; charset=UTF-8\n\n退室処理失敗 - ユーザーIDが不整合です．" );
		exit;
	}

	# ◆切断済みは退室できない
	if ( $data->{'users'}->{$id}->{'name'} eq '◆切断済み' )
	{
		print( "Content-type: text/plain; charset=UTF-8\n\n退室処理失敗 - 切断済みです．ブラウザでリロードしてください．" );
		exit;
	}

	# ◆閲覧者は退室できない
	if ( $data->{'users'}->{$id}->{'name'} eq '◆閲覧者' ) 
	{
		print( "Content-type: text/plain; charset=UTF-8\n\n退室処理失敗 - まだ入室されていません．" );
		exit;
	}

	# 退室メッセージ書き込み
	{
		my $newMessage = {
			'speaker' => "<span title=\"UA: $ENV{'HTTP_USER_AGENT'} , Host: $ENV{'REMOTE_HOST'}\">$data->{'users'}->{$id}->{'name'}</span>" ,
			'statement' => '<span class="system_message">退室</span>' ,
			'postedAt' => &string::yymmddhhMMss()
		};
		push( @{ $data->{'conversation'} } , $newMessage );
	}

	# ユーザーを◆閲覧者に
	$data->{'users'}->{$id}->{'name'} = '◆閲覧者';

	# 更新時刻書き換え
	$data->{'lastUpdate'} = $now;

	print( "Content-type: text/plain; charset=UTF-8\n\n退室処理完了" );
	exit;
}



# リロード
if ( $argument{'mode'} eq 'reload' )
{
	my $startedAt = time();

	while ( $startedAt + $main::configuration{'reconnectTime'} > time() )
	{
		{
			if ( my $data = new Data( $main::configuration{'dataFile'} ) )
			{
				my $now = Time::HiRes::time();

				# 新規接続の場合は◆閲覧者を一人増やし送信
				if ( !exists( $data->{'users'}->{$id} ) )
				{
					$data->{'users'}->{$id}->{'name'} = '◆閲覧者';
					$data->{'lastUpdate'} = $now;
					$data->{'users'}->{$id}->{'lastAccess'} = $now;
					$data->sendData();
					exit;
				}

				# ◆切断済みは閲覧できない
				if ( $data->{'users'}->{$id}->{'name'} eq '◆切断済み' )
				{
					print( "Content-type: text/plain; charset=UTF-8\n\n更新処理失敗 - 切断済みです．ブラウザでリロードしてください．" );
					exit;
				}

				# 更新があれば送信
				if ( 0 + $data->{'lastUpdate'} > 0 + $data->{'users'}->{$id}->{'lastAccess'} )
				{
					$data->{'users'}->{$id}->{'lastAccess'} = $now;
					$data->sendData();
					exit;
				}
				$data->{'users'}->{$id}->{'lastAccess'} = $now;
			}
		}
		Time::HiRes::sleep( $main::configuration{'interval'} );
	}
	print( "Content-type: text/plain; charset=UTF-8\n\n更新処理完了 - 更新はありません．" );
	exit;
}



# 投稿
if ( $argument{'mode'} eq 'post' )
{
	my $data;
	my $now = Time::HiRes::time();

	# メッセージ確認
	if ( !$argument{'statement'} )
	{
		print( "Content-type: text/plain; charset=UTF-8\n\n投稿処理失敗 - 文章が入力されていません．" );
		exit;
	}

	# データ読み込み
	{
		my $count = 0;
		while ( !( $data = new Data( $main::configuration{'dataFile'} ) ) )
		{
			Time::HiRes::sleep( $main::configuration{'lockInterval'} );
			if ( ++$count > $main::configuration{'retry'} )
			{
				print( "Content-type: text/plain; charset=UTF-8\n\n投稿処理失敗 - ロックができませんでした．" );
				exit;
			}
		}
	}

	# ユーザーID確認
	if ( !exists( $data->{'users'}->{$id} ) )
	{
		print( "Content-type: text/plain; charset=UTF-8\n\n投稿処理失敗 - ユーザーIDが不整合です．" );
		exit;
	}

	# ◆切断済みは投稿できない
	if ( $data->{'users'}->{$id}->{'name'} eq '◆切断済み' )
	{
		print( "Content-type: text/plain; charset=UTF-8\n\n投稿処理失敗 - 切断済みです．ブラウザでリロードしてください．" );
		exit;
	}

	# ◆閲覧者は投稿できない
	if ( $data->{'users'}->{$id}->{'name'} eq '◆閲覧者' ) 
	{
		print( "Content-type: text/plain; charset=UTF-8\n\n投稿処理失敗 - まだ入室されていません．" );
		exit;
	}

	# メッセージ書き込み
	{
		my $newMessage = {
			'speaker' => "<span title=\"UA: $ENV{'HTTP_USER_AGENT'} , Host: $ENV{'REMOTE_HOST'}\">$data->{'users'}->{$id}->{'name'}</span>" ,
			'statement' => $argument{'statement'} ,
			'postedAt' => &string::yymmddhhMMss()
		};
		push( @{ $data->{'conversation'} } , $newMessage );
	}

	# 更新時刻書き換え
	$data->{'lastUpdate'} = $now;

	print( "Content-type: text/plain; charset=UTF-8\n\n投稿処理完了" );
	exit;
}



# 切断
if ( $argument{'mode'} eq 'disconnect' )
{
	my $data;

	# データ読み込み
	{
		my $count = 0;
		while ( !( $data = new Data( $main::configuration{'dataFile'} ) ) )
		{
			Time::HiRes::sleep( $main::configuration{'lockInterval'} );
			if ( ++$count > $main::configuration{'retry'} )
			{
				print( "Content-type: text/plain; charset=UTF-8\n\n切断処理失敗 - ロックができませんでした．" );
				exit;
			}
		}
	}

	# ユーザーID確認
	if ( !exists( $data->{'users'}->{$id} ) )
	{
		print( "Content-type: text/plain; charset=UTF-8\n\n切断処理失敗 - ユーザーIDが不整合です．" );
		exit;
	}

	# ◆切断済みは切断できない
	if ( $data->{'users'}->{$id}->{'name'} eq '◆切断済み' )
	{
		print( "Content-type: text/plain; charset=UTF-8\n\n切断処理失敗 - 切断済みです．ブラウザでリロードしてください．" );
		exit;
	}

	# ◆閲覧者でなければ退室メッセージ書き込み
	if ( $data->{'users'}->{$id}->{'name'} ne '◆閲覧者' ) 
	{
		my $newMessage = {
			'speaker' => "<span title=\"UA: $ENV{'HTTP_USER_AGENT'} , Host: $ENV{'REMOTE_HOST'}\">$data->{'users'}->{$id}->{'name'}</span>" ,
			'statement' => '<span class="system_message">退室</span>' ,
			'postedAt' => &string::yymmddhhMMss()
		};
		push( @{ $data->{'conversation'} } , $newMessage );
	}

	# ユーザー無効化
	$data->{'users'}->{$id}->{'name'} = '◆切断済み';

	# 更新時刻書き換え
	$data->{'lastUpdate'} = Time::HiRes::time();

	print( "Content-type: text/plain; charset=UTF-8\n\n切断処理完了" );
	exit;
}


print( "Content-type: text/plain; charset=UTF-8\n\nパラメータが不正です．" );
exit;
