


use Time::HiRes;

require 'configuration.pl';
require 'string.pl';
require 'lock.pl';



package Data;



# データを読み込みます
# 引数 1  なし
# 返り値  データ（連想配列）
sub new
{
	my $this = $_[0];
	my $now = Time::HiRes::time();
	my $users = '';
	my $conversation = '';
	my $data = {
		'lock' =>  ,
		'lastUpdate' => '' ,
		'users' => {} ,
		'conversation' => []
	};

	# ロック
	if ( !( $data->{'lock'} = new Lock() ) )
	{
		return 0;
	}

	# データ読み込み
	{
		my $buffer = '';
		if ( open( FILE , $main::configuration{'dataFile'} ) )
		{
			read( FILE , $buffer , -s $main::configuration{'dataFile'} );
			close(FILE);
		}
		( $data->{'lastUpdate'} , $users , $conversation ) = split( "\n<&&>\n" , $buffer );
	}

	# 会話リスト作成
	foreach ( split( "\n" , $conversation ) )
	{
		my $newMessage = {};
		( $newMessage->{'speaker'} , $newMessage->{'statement'} , $newMessage->{'postedAt'} ) = split( '<>' , $_ );
		if ( $newMessage->{'speaker'} )
		{
			push( @{ $data->{'conversation'} } , $newMessage );
		}
	}

	# ユーザーリスト作成
	foreach ( split( "\n" , $users ) )
	{
		my ( $id , $name , $lastAccess ) = split( '<>' , $_ );

		# 時間切れなら除外
		if ( 0 + $lastAccess + $main::configuration{'logOffTime'} <= $now )
		{
			# 入室者なら時間切れメッセージ表示
			if ( $name ne '◆閲覧者' and $name ne '◆切断済み' )
			{
				$newMessage = {
					'speaker' => $name ,
					'statement' => '<span class="system_message">接続期限切れ退室</span>' ,
					'postedAt' => &string::yymmddhhMMss( $now )
				};
				push( @{ $data->{'conversation'} } , $newMessage );
			}
			$data->{'lastUpdate'} = $now;
		}

		# 時間切れでなければリストに要素を追加
		else
		{
			my $newId = { 'name' => $name , 'lastAccess' => $lastAccess };
			$data->{'users'}->{$id} = $newId;
		}
	}
	return bless( $data , $this );
}



# データを書きこみます
# 引数 1  なし
# 返り値  なし
sub DESTROY
{
	my $this = $_[0];
	my $isNewFile = 0;

	# データを開く
	if ( !( -e $main::configuration{'dataFile'} ) )
	{
		$isNewFile = 1;
	}
	open( FILE , '>' , $main::configuration{'dataFile'} );

	# 更新時刻書き込み
	print( FILE "$this->{'lastUpdate'}\n<&&>\n" );

	# ユーザー情報があれば書き込む
	if ( keys( %{ $this->{'users'} } ) )
	{
		foreach ( keys( %{ $this->{'users'} } ) )
		{
			print( FILE $_ . '<>' . $this->{'users'}->{$_}->{'name'} . '<>' . $this->{'users'}->{$_}->{'lastAccess'} . "\n" );
		}
	}

	# ユーザー情報が空なら改行だけ
	else
	{
		print( FILE "\n" );
	}

	print( FILE '<&&>' );

	# 会話があれば書き込む
	if ( @{ $this->{'conversation'} } )
	{
		# 会話の数が多い場合はログへ移動
		if ( @{ $this->{'conversation'} } > $main::configuration{'maxMessage'} )
		{
			my $isNewFile = 0;
			if ( !( -e $main::configuration{'logFile'} ) )
			{
				$isNewFile = 1;
			}

			open( LOG_FILE , '>>' , $main::configuration{'logFile'} );
			my $count = @{ $this->{'conversation'} } - $main::configuration{'maxShowMessage'};
			while ( $count-- )
			{
				my $it = shift( @{ $this->{'conversation'} } );
				print( LOG_FILE $it->{'speaker'} . '<>' . $it->{'statement'} . '<>' . $it->{'postedAt'} . '<>' . $it->{'remoteHost'} . "\n" );
			}
			close(LOG_FILE);

			if ( $isNewFile )
			{
				chmod( $main::configuration{'logFile'} , $main::configuration{'isSuExec'} ? 0600 : 0606 );
			}
		}

		# 書き込み
		foreach ( @{ $this->{'conversation'} } )
		{
			print( FILE "\n" . $_->{'speaker'} . '<>' . $_->{'statement'} . '<>' . $_->{'postedAt'} );
		}
	}

	# 会話が空なら改行だけ
	else
	{
		print( FILE "\n" );
	}

	close(FILE);

	# ファイルを新たに作成した場合は，アクセス権を適切に設定
	if ( $isNewFile )
	{
		chmod( $main::configuration{'dataFile'} , $main::configuration{'isSuExec'} ? 0600 : 0606 );
	}
}



# データをブラウザに送信します
# 引数 1  なし
# 返り値  なし
sub sendData
{
	my $this = $_[0];
	my $rom = 0;

	print( "Content-type: text/plain; charset=UTF-8\n\n" );

	# ユーザー名出力
	print( "$this->{'users'}->{$main::id}->{'name'}<>\n" );

	# ユーザーリスト出力
	foreach ( keys( %{ $this->{'users'} } ) )
	{
		# ◆閲覧者カウント
		if ( $this->{'users'}->{$_}->{'name'} eq '◆閲覧者' )
		{
			++$rom;
		}
		# ◆切断済みでなければ表示
		elsif ( $this->{'users'}->{$_}->{'name'} ne '◆切断済み' )
		{
			print( "\n\t\t\t\t<li>$this->{'users'}->{$_}->{'name'}</li>" );
		}
	}
	print( << "END" );

				<li class="rom">◆閲覧者 $rom 人</li>
			<>

			<table>
				<tbody>
END
	print( "\t\t\t\t" );

	# 会話があれば出力
	if ( @{ $this->{'conversation'} } )
	{
		my $i = @{ $this->{'conversation'} };
		my $end = @{ $this->{'conversation'} } - $main::configuration{'maxShowMessage'};
		if ( $end < 0 )
		{
			$end = 0;
		}

		# 最近の投稿が上になるように，逆順で出力
		while ( --$i >= $end )
		{
			print( << "END" );
	<tr>
						<td class="speaker">$this->{'conversation'}->[$i]->{'speaker'}</td>
						<td class="statement">$this->{'conversation'}->[$i]->{'statement'}</td>
						<td class="posted_at">$this->{'conversation'}->[$i]->{'postedAt'}</td>
					</tr>
END
			print( "\t\t\t\t");
		}
	}

	# 会話が空なら空セルだけ
	else
	{
		print( << "END" );
	<tr>
						<td class="speaker"></td>
						<td class="statement"></td>
						<td class="posted_at"></td>
					</tr>
END
		print( "\t\t\t\t");
	}
	print( "</tbody>\n\t\t\t</table>\n\t\t" );
}



1;
