


use strict;

use Jcode;
use Digest::SHA1;



package string;



# QUERY_STRINGをハッシュに変換します．
# 引数 1  元になる文字列
# 返り値  ハッシュ
sub splitQueryString
{
	my $input = $_[0];
	my %result = ();

	foreach ( split( '&' , $input ) )
	{
		my ( $key , $value ) = split( '=' , $_ );

		$value =~ s/\+/ /g;
		$value =~ s/%[01][a-fA-F0-9]/ /g;
		$value =~ s/%([a-fA-F2-9][a-fA-F0-9])/pack( 'H2' , $1 )/eg;

		$value =~ s/&/&amp;/g;
		$value =~ s/</&lt;/g;
		$value =~ s/>/&gt;/g;
		$value =~ s/"/&quot;/g;
		$value =~ s/'/&#39;/g;

		$result{ $key } = $value;
	}
	return %result;
}



# UNIX時刻を現地時刻の文字列に変換します 
# 引数 1  変換元のUNIX時刻．指定しなければ現在時刻．
# 返り値  変換後の文字列
sub yymmddhhMMss
{
	my ( $sec , $min , $hour , $mday , $mon , $year , $wday , $yday , $isdst ) = localtime( $_[0] ? $_[0] : time() );
	return sprintf( "<span title=\"%d/%02d/%02d %02d:%02d:%02d\">%02d:%02d:%02d</span>" , $year + 1900 , $mon + 1 , $mday , $hour , $min , $sec , $hour , $min , $sec );
}



# 名前をトリップ化します．
# 引数 1  変換前の名前
# 返り値  変換後の名前
sub trip
{
	my $name = $_[0];
	$name =~ tr/◆/◇/;

	if ( $name =~ /(.*?)#(.*)/ )
	{
		my @splitName = split( '#' , $name );
		my $base = $1;
		my $key = $2;
		Jcode::convert( \$key , 'sjis' , 'utf8' );
		my $trip;

		if ( length( $key ) >= 12)
		{
			my $mark = substr( $key , 0 , 1 );
			if ($mark eq '#' || $mark eq '$')
			{
				if ( $key =~ m|^#([[:xdigit:]]{16})([./0-9A-Za-z]{0,2})$| )
				{
					$trip = substr( crypt( pack( 'H*' , $1 ) , "$2.." ) , -10 );
				}
				else
				{
					$trip = '???';
				}
			}
			else
			{
				$trip = substr( Digest::SHA1::sha1_base64( $key ) , 0 , 12 );
				$trip =~ tr/+/./;
			}
		}
		else
		{
			my $salt = substr( $key . 'H.' , 1 , 2 );
			$salt =~ s/[^\.-z]/\./go;
			$salt =~ tr/:;<=>?@[\\]^_`/ABCDEFGabcdef/;
			$trip = substr( crypt( $key , $salt ) , -10 );
		}
		$name = $base . '◆' . $trip;
	}

	return $name;
}



1;
